// This program builds static resources out of the files in the
// public folder to be served. It reads the name of the public
// folder from the static-publish.rc.js file.
// With create-react-app, this would be the ./build directory.
// This also reexports the "spa" value in the static-publish.rc.js
// file so that the C@E handler knows what file to serve up if
// the resource doesn't map to a file.
import * as fs from "fs";
import * as path from "path";
import * as url from "url";
function getFiles(results, dir) {
    const entries = fs.readdirSync(dir, { withFileTypes: true });
    for (const entry of entries) {
        const name = path.resolve(dir, entry.name);
        if (entry.isDirectory()) {
            getFiles(results, name);
        }
        else {
            results.push(name);
        }
    }
}
export async function buildStaticLoader(commandLineValues) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    const { 'suppress-framework-warnings': suppressFrameworkWarnings } = commandLineValues;
    const displayFrameworkWarnings = !suppressFrameworkWarnings;
    console.log("🚀 Building loader...");
    let config;
    try {
        const staticPublishRcPath = path.resolve('./static-publish.rc.js');
        config = (await import(staticPublishRcPath)).default;
    }
    catch (ex) {
        console.error("❌ Can't load static-publish.rc.js");
        console.error("Run this from a compute-js-static-publish compute-js directory.");
        console.error("Error: ", String(ex));
        process.exitCode = 1;
        return;
    }
    const outputDir = path.resolve();
    const publicDirRoot = path.resolve((_a = config.publicDir) !== null && _a !== void 0 ? _a : './');
    const results = [];
    getFiles(results, publicDirRoot);
    console.log(`✔️ Public directory '${publicDirRoot}'.`);
    const staticDirs = (_b = config.staticDirs) !== null && _b !== void 0 ? _b : [];
    if (staticDirs.length > 0) {
        console.log(`✔️ Using static directories: ${staticDirs.join(', ')}`);
    }
    else {
        if (displayFrameworkWarnings) {
            console.log(`✔️ No static directories defined.`);
        }
    }
    const staticRoots = staticDirs.map(dir => path.resolve(publicDirRoot, dir));
    const DEFAULT_EXCLUDE_DIRS = [
        './node_modules'
    ];
    const excludeDirs = (_c = config.excludeDirs) !== null && _c !== void 0 ? _c : DEFAULT_EXCLUDE_DIRS;
    if (excludeDirs.length > 0) {
        console.log(`✔️ Using exclude directories: ${excludeDirs.join(', ')}`);
    }
    else {
        if (displayFrameworkWarnings) {
            console.log(`✔️ No exclude directories defined.`);
        }
    }
    const excludeRoots = excludeDirs.map(dir => path.resolve(publicDirRoot, dir));
    // Load defaultContentTypes module
    const __dirname = url.fileURLToPath(new URL('.', import.meta.url));
    const defaultContentTypesJsSrcPath = path.resolve(__dirname, '../resources/default-content-types.cjs');
    const defaultContentTypes = await import(defaultContentTypesJsSrcPath);
    // Load content types
    const finalContentTypes = defaultContentTypes.mergeContentTypes((_d = config.contentTypes) !== null && _d !== void 0 ? _d : []);
    const DEFAULT_INCLUDE_DIRS = [
        './.well-known'
    ];
    const includeDirs = (_e = config.includeDirs) !== null && _e !== void 0 ? _e : DEFAULT_INCLUDE_DIRS;
    if (includeDirs.length > 0) {
        console.log(`✔️ Using include directories: ${includeDirs.join(', ')}`);
    }
    else {
        if (displayFrameworkWarnings) {
            console.log(`✔️ No include directories defined.`);
        }
    }
    const includeRoots = includeDirs.map(dir => path.resolve(publicDirRoot, dir));
    const excludeTest = config.excludeTest;
    const files = results
        .filter(file => {
        // Exclude files that come from C@E app dir
        if (outputDir != publicDirRoot && file.startsWith(outputDir)) {
            return false;
        }
        // Include files that come from "included roots" dir
        if (includeRoots.some(root => file.startsWith(root))) {
            return true;
        }
        // Exclude files that are in directories that start with "."
        if (file.indexOf('/.') !== -1) {
            return false;
        }
        // Exclude files that come from "excluded roots" dir
        if (excludeRoots.some(root => file.startsWith(root))) {
            return false;
        }
        return true;
    })
        .filter(file => {
        return excludeTest == null || !excludeTest(file);
    });
    const moduleTest = config.moduleTest;
    const knownAssets = {};
    // Create "static content" dir that will be used to hold a copy of static files
    // NOTE: this is needed because includeBytes doesn't seem to be able to traverse
    // up to parent dir of Compute project.
    const staticContentDir = './src/static-content';
    fs.rmSync(staticContentDir, { recursive: true, force: true });
    fs.mkdirSync(staticContentDir, { recursive: true });
    let fileContents = '';
    fileContents += 'import { StaticAssets } from "@fastly/compute-js-static-publish";\n';
    fileContents += 'import { includeBytes } from "fastly:experimental";\n';
    // If any files need to be loaded as modules, import them now.
    const loadAsModule = [];
    if (moduleTest != null) {
        for (const [index, file] of files.entries()) {
            const relativeFilePath = path.relative('./src', file);
            const filePath = file.slice(publicDirRoot.length);
            if (moduleTest(filePath)) {
                loadAsModule[index] = true;
                fileContents += `import * as fileModule${index} from "${relativeFilePath}";\n`;
            }
        }
    }
    fileContents += 'const textDecoder = new TextDecoder();\n';
    fileContents += `\nexport const assets = {\n`;
    for (const [index, file] of files.entries()) {
        const contentDef = defaultContentTypes.testFileContentType(finalContentTypes, file);
        const filePath = file.slice(publicDirRoot.length);
        const type = contentDef === null || contentDef === void 0 ? void 0 : contentDef.type;
        const isStatic = staticRoots.some(root => file.startsWith(root));
        if (contentDef != null) {
            console.log('✔️ ' + filePath + ': ' + JSON.stringify(type) + (isStatic ? ' [STATIC]' : ''));
        }
        else {
            if (displayFrameworkWarnings) {
                console.log('⚠️ Notice: Unknown file type ' + filePath + '. Treating as binary file.');
            }
        }
        // Serve static files by copying them into the static content dir and using includeBytes()
        let staticFileExt, staticFileTypeDesc, isBinary;
        if (contentDef == null || contentDef.binary) {
            staticFileExt = '.bin';
            staticFileTypeDesc = 'binary';
            isBinary = true;
        }
        else {
            staticFileExt = '.txt';
            staticFileTypeDesc = 'text';
            isBinary = false;
        }
        const staticFilePath = `${staticContentDir}/file${index}${staticFileExt}`;
        fs.cpSync(file, staticFilePath);
        console.log(`✔️ Copied static ${staticFileTypeDesc} file "${file}" to "${staticFilePath}".`);
        let content = `includeBytes("${staticFilePath}")`;
        if (!isBinary) {
            content = `textDecoder.decode(${content})`;
        }
        let module;
        if (loadAsModule[index]) {
            module = 'fileModule' + index;
        }
        else {
            module = 'null';
        }
        fileContents += `  ${JSON.stringify(filePath)}: { contentType: ${JSON.stringify(type)}, content: ${content}, module: ${module}, isStatic: ${JSON.stringify(isStatic)} },\n`;
    }
    fileContents += '};\n';
    let spaFile = (_f = config.spa) !== null && _f !== void 0 ? _f : false;
    if (spaFile) {
        console.log(`✔️ Application SPA file '${spaFile}'.`);
        if (!knownAssets[spaFile] || knownAssets[spaFile].contentType !== 'text/html') {
            if (displayFrameworkWarnings) {
                console.log(`⚠️ Notice: '${spaFile}' does not exist or is not of type 'text/html'. Ignoring.`);
            }
            spaFile = false;
        }
    }
    else {
        if (displayFrameworkWarnings) {
            console.log(`✔️ Application is not a SPA.`);
        }
    }
    fileContents += `\nexport const spaFile = ${JSON.stringify(spaFile)};\n`;
    let notFoundPageFile = (_g = config.notFoundPage) !== null && _g !== void 0 ? _g : false;
    if (notFoundPageFile) {
        console.log(`✔️ Application 'not found (404)' file '${notFoundPageFile}'.`);
        if (!knownAssets[notFoundPageFile] || knownAssets[notFoundPageFile].contentType !== 'text/html') {
            if (displayFrameworkWarnings) {
                console.log(`⚠️ Notice: '${notFoundPageFile}' does not exist or is not of type 'text/html'. Ignoring.`);
            }
            notFoundPageFile = false;
        }
    }
    else {
        if (displayFrameworkWarnings) {
            console.log(`✔️ Application specifies no 'not found (404)' page.`);
        }
    }
    fileContents += `\nexport const notFoundPageFile = ${JSON.stringify(notFoundPageFile)};\n`;
    let autoIndex = (_h = config.autoIndex) !== null && _h !== void 0 ? _h : null;
    fileContents += `\nexport const autoIndex = ${JSON.stringify(autoIndex)};\n`;
    let autoExt = (_j = config.autoExt) !== null && _j !== void 0 ? _j : null;
    fileContents += `\nexport const autoExt = ${JSON.stringify(autoExt)};\n`;
    fileContents += '\nexport const staticAssets = new StaticAssets(assets);\n';
    fs.writeFileSync('./src/statics.js', fileContents);
    console.log("✅  Wrote static file loader for " + files.length + " file(s).");
}
